;+
;   name:
;        bytecube.pro
;
;   purpose:
;	a function that bytscls a datacube frame by frame, making 
;	something that (hopefully) looks decent when animated using
;	mov3r.pro.  (Don't use with xmov.pro or xinteranimate.pro, 
;	as these programs will rescale the images again, making a 
;	big mess.)
;
;   calling sequence:
;       result = bytecube(incube,rng=rng,abs=abs) 
;
;   inputs:
;	incube = a datacube of any non-string data type
;
;   keywords:
;	rng = two element vector with number of standard deviations
;		to scale each image to.  (ie. [min,max])
;	abs = set to scale all images to mean and standard deviation
;		of first frame.
;
;   output:
;       a byte array of same dimensions as incube
;
;   history:
;       written by R. Gutermuth - 5-25-01
;       8-29-01 - added ability to use a mask vector (or array of same). (RAG)
;
;-


function bytecube,incube,mask=mask,rng=rng,abs=abs

sz = size(incube)

if sz[0] ne 3 then begin
 print,'ERROR in bytecube.pro: not a data cube!!'
 print,strtrim(sz[0],2)
 return,[-1]
endif

if keyword_set(mask) then begin
 msz = size(mask)
 if msz[0] eq 2 then begin
  if msz[2] ne sz[3] then begin
   print,'ERROR in bytecube.pro: Invalid mask array'
   return,-1
  endif
  flg = 1
 endif else flg = 0
endif
   
result = bytarr(sz[1],sz[2],sz[3],/nozero)

for i = 0,sz[3]-1 do begin
 
 im = incube[*,*,i]
 med = median(im)

 if keyword_set(mask) then begin
  if flg eq 1 then begin
   if mask[0,i] eq 0 then begin
    w = where(mask[*,i] ne 0,ct)
    tmpmask = lonarr(ct+1)
    tmpmask[1:ct] = (mask[*,i])[w]
   endif else tmpmask = (mask[*,i])[where(mask[*,i] ne 0)]
  endif else if i eq 0 then tmpmask = mask
  im[tmpmask] = med
 endif 

 if keyword_set(abs) then begin
  if i eq 0 then begin
   mo = moment(im,sdev=sd)
   mid = med
  endif
 endif else begin
  mo = moment(im,sdev=sd)
  mid = med
 endelse

 if not keyword_set(rng) then rng = [-2,2]

 imb = bytscl(im,min=mid+rng[0]*sd,max=mid+rng[1]*sd)

 result[*,*,i] = imb

endfor

return,result

end 


