;+
;
; radial_plot1 - makes a radial distribution plot from a PVDAT file.
;
; CALLING SEQUENCE:
;
;  radial_plot1,infile=infile,pix2pc=pix2pc,bin_size=bin_size,maxdist=maxdist,
;               center=center,maglimit=maglimit,title=title,xtitle=xtitle,
;               ytitle=ytitle,charsize=charsize
;
;-

pro radial_plot1,infile=infile,pix2pc=pix2pc,bin_size=bin_size,maxdist=maxdist,$
                 center=center,maglimit=maglimit,title=title,xtitle=xtitle,$
                 ytitle=ytitle,charsize=charsize,outdata=outdata,$
                 overplot=overplot

 if not keyword_set(infile) then begin
  infile = dialog_pickfile(title='Select PVDAT file',/must_exist)
  if infile eq '' then begin
   print,'Cancelled...'
   return
  endif
 endif else begin
  it = size(infile,/type)
  if it eq 7 then restore,infile else pv_dat = infile
 endelse

 if not keyword_set(maglimit) then maglimit = 25.

 w = where(pv_dat[7,*] lt maglimit)

 if not keyword_set(bin_size) then bin_size = 20.

 if not keyword_set(pix2pc) then begin
  labtxt1 = 'pix'
  pix2pc = 1.
 endif else labtxt1 = 'pc'

 if not keyword_set(center) then begin
  center = [median(pv_dat[0,w]),median(pv_dat[1,w])]
 endif

 if not keyword_set(maxdist) then begin
  maxdist = min([[max(pv_dat[0,w]),max(pv_dat[1,w])]-center,center])
  maxdist = fix(maxdist / float(bin_size)) * bin_size
 endif

 dist = sqrt((pv_dat[0,w]-center[0])^2 + (pv_dat[1,w]-center[1])^2)

 dhist = [0,histogram([0,reform(dist)],bin=bin_size)]

 dhist[1]=dhist[1]-1

 bins = (findgen(n_elements(dhist))-.5) * bin_size

 area = !pi * ((bins + .5 * bin_size)^2 - (bins - .5 * bin_size)^2)

 outdata = fltarr(2,n_elements(dhist))
 outdata[0,*] = bins*pix2pc
 outdata[1,*] = dhist / area / pix2pc^2

 if not keyword_set(overplot) then begin
  plot,outdata[0,*],outdata[1,*],psym=10,xran=[0,maxdist * pix2pc],/xstyle,$
       title=title,xtitle=xtitle,ytitle=ytitle,charsize=charsize
 endif else begin
  oplot,outdata[0,*],outdata[1,*],psym=10,linestyle=2
 endelse

end
