;+
;  name: 
;         xmedian_stack.pro
;
;  purpose:
;         Median filters a stack of images.  (Wanna ignore some pixels?
;         Set them to NaN and the IDL median function ignores them!)
;
;  calling sequence:
;         result = xmedian_stack(datacube)
;
;  inputs:
;         datacube = pointer to stack of data : 3d data cube [k,i,j]
;
;  keywords:
;         NONE
;
;  output:
;         result = a pointer to a 2d image containing the median
;                pixel values 
;
;  called programs:
;         NONE
;
;  history:  
;         written by S. T. Megeath - summer 1998 
;         last modified 12-28-98
;         2-12-02 - major modifications: now expecting pointer to
;                   datacube which we will median along the X
;                   direction...  also indexed things to follow 
;                   vector order of array. (RAG)
;
;-

function xmedian_stack,datacube

;find the size of one image and create final data array
 sz = size(*datacube)
 med = ptr_new(make_array(sz[2],sz[3],type=sz[sz[0]+1],/nozero))

;for every pixel in the data array, calculate the median
 for j = 0, sz[3]-1 do for i = 0, sz[2]-1 do (*med)[i,j] = $
    median((*datacube)[*,i,j],/even)
 return,med

end


