;-------------------------------------------------------------
;+
; NAME:
;      BOX_PLOT
; PURPOSE:
;      This procedure, given two x values and two y values
;      will draw a rectangular box in between those points.
;      Neither the points x1 and x2 nor the points y1 and
;      y2 need to be increasing.
; CATEGORY:
;      DISPLAY
; CALLING SEQUENCE:
;      box_plot,x1,x2,y1,y2,[clr]
; INPUTS:
;      x1 = The first x position.
;        type: scalar,integer
;      x2 = The second x position.
;        type: scalar,integer
;      y1 = The first y position.
;        type: scalar,integer
;      y2 = The second y position.
;        type: scalar,integer
;      clr = The color you want to use if you are using device
;        coordinates. If you are not using device coordinates
;        then you need not enter in a value for clr.
;        type: scalar,integer,0 to 255
; KEYWORD PARAMETERS:
;      /NORMAL = Normalized coordinates.
;      /DATA = Data coordinates.
;      /DEVICE = Device coordinates.
;      THICK = PLOT keyword thick.
;        (Default = 1.)
; OUTPUTS:
;      A Rectangular box printed out on the current graphics device.
; COMMON BLOCKS:
; NOTES:
; MODIFICATION HISTORY:
;      H. Cohl,  20 Jul, 1992 ---  Fixed bug that caused box to
;                                  be drawn in false coordinate system.
;      H. Cohl,  19 Sep, 1991 ---  Fixed bugs that Steve Keil
;                                  pointed out.
;      H. Cohl,  10 Nov, 1990 ---  Initial Programming.
;-
;-------------------------------------------------------------

pro box_plot,x1,x2,y1,y2,clr,normalized=normalized,data=data,device=device,$
             help=help,thick=thick

  ;Display idl header if help is required.
  if keyword_set(help) or n_params() lt 4 then begin
    dl,'box_plot.pro'
    goto,finishup
  endif

  if (x2 lt x1) then begin a=x1 & x1=x2 & x2=a & endif
  if (y2 lt y1) then begin b=y1 & y1=y2 & y2=b & endif

  if n_elements(thick) eq 0 then thick=1
  if n_elements(clr) eq 0 then begin
    if keyword_set(device) then begin
      ;show the box on display
      plots,[x1,x2,x2,x1,x1],[y1,y1,y2,y2,y1],/dev,thick=thick
    endif else if keyword_set(normalized) then begin
      ;show the box on display
      plots,[x1,x2,x2,x1,x1],[y1,y1,y2,y2,y1],/norm,thick=thick
    endif else if keyword_set(data) then begin
      ;show the box on display
      plots,[x1,x2,x2,x1,x1],[y1,y1,y2,y2,y1],/dat,thick=thick
    endif else begin
      ;show the box on display
      plots,[x1,x2,x2,x1,x1],[y1,y1,y2,y2,y1],/dev,thick=thick
    endelse
  endif else begin
    if keyword_set(device) then begin
      ;show the box on display
      plots,[x1,x2,x2,x1,x1],[y1,y1,y2,y2,y1],/dev,color=clr,$
            thick=thick
    endif else if keyword_set(normalized) then begin
      ;show the box on display
      plots,[x1,x2,x2,x1,x1],[y1,y1,y2,y2,y1],/norm,color=clr,$
          thick=thick
    endif else if keyword_set(data) then begin
      ;show the box on display
      plots,[x1,x2,x2,x1,x1],[y1,y1,y2,y2,y1],/dat,color=clr,$
            thick=thick
    endif else begin
      ;show the box on display
      plots,[x1,x2,x2,x1,x1],[y1,y1,y2,y2,y1],/dev,color=clr,$
            thick=thick
    endelse
  endelse

  finishup:

end
