;...............................................................
;
; 	integrate cube in v
;
;       Parameters:  cube - datacube
;		     vel  - vector of velocity values
;                    vel1 - starting velocity
;		     vel2 - ending velocity
;		     order - order of axes (default: vxy)
;		     moment - use moment method (not implemented)
;		     kernel - kernel for moment method
;		     clip - clipping value for data
;
;       1-18-99 S. T. Megeath
;...............................................................

function int_slice,cube,vel,vel1=vel1,vel2=vel2,order=order,moment=moment,$
                   kernel=kernel,clip=clip

  if NOT keyword_set(order) then order = "vxy"

 
; create mask for moment method application
  if keyword_set(moment) then begin
    mask = cube
    if NOT keyword_set(kernel) then kernel = replicate(1,1,3,3)
    sum_kernal = total(total(total(kernal,1),1),1)
    mask = convol(mask,kernel,sum_kernal)
    mask = mask ge moment
    print,"MOMENT METHOD"
  endif else begin
     mask = 0.*cube+1.
  endelse

  if keyword_set(clip) then begin
    data = cube * (cube gt clip)
  endif

  if (NOT keyword_set(clip) and NOT keyword_set(moment)) then begin
     data = cube
  endif

  sum = 0
  diff = (vel[1]-vel[0])
  nchan = n_elements(vel)
  channel1 = 0
  channel2 = 0

  channel1 = 0 > fix((vel1-vel[0])/diff+.5) < (nchan-1)
  channel2 = 0 > fix((vel2-vel[0])/diff+.5) < (nchan-1)

; print,channel1,channel2,fix((vel1-vel[0])/diff+.5),fix((vel2-vel[0])/diff+.5)

  chmin = channel1 < channel2
  chmax = channel2 > channel1

; print,chmin,chmax,nchan

;depending on order of x, y, and v, sum over all velocity channels
  case order of
    "vxy": sum = total(data[chmin:chmax,*,*]*mask[chmin:chmax,*,*],1)
    "xvy": sum = total(data[*,chmin:chmax,*]*mask[*,chmin:chmax,*],2)
    "xyv": sum = total(data[*,*,chmin:chmax]*mask[*,*,chmin:chmax],3)
  endcase
  
;convert units to T_a * dv
  sum = sum*abs(diff)
  print,"max sum =",max(sum)
return,sum
end

