;..........................................................................
;
;   lbcube: this function returns an 2 x n array of all the positions (l,b)
;           in a datacube with spectra.
;
;	    Note: this only works on cubes in an "vxy" format.
;
;   parameters:   cube - data cube in "vxy" format
;                 l    - vector containing longitude as a function of pixel
;                 b    - vector containing lattitude as a function of pixel
;
;   1-20-99: S. T. Megeath
;..........................................................................

function lbcube,datacube,l,b

; determine size of array
  xpix = n_elements(l)
  ypix = n_elements(b)

; create and initialize variables
  pos = fltarr(2,xpix*ypix)
  count = 0

; go through 
  cube = datacube*(datacube gt -100)
  for i = 0,xpix-1 do begin 
    for j = 0,ypix-1 do begin
      sdev = total(cube[*,i,j]*cube[*,i,j],1)+$
             total(cube[*,i,j],1)*total(cube[*,i,j],1)
      sdev = sqrt(sdev)/n_elements(cube[*,i,j])
      if (sdev gt 0.0) then begin
        pos[*,count] = [l[i],b[j]]
        count = count+1
      endif
    endfor
  endfor

  return,pos

end
