;----------------------------------------------------------------
;
;	cluster_plot
;
;	this program takes a list (ra,dec) positions used to 
;	generate a cluster AOR, and plots these onto the displayed
;	image.
;
;	parameter: rot  - rotation angle of the spacecraft
;		
;	keywords:  fov  - which fov to use, default "fov1"
;		   infile - input data from file infile
;		   data   - input data from array data
;
;	NOTE: if both infile and data keywords are set, then
;	      this program will input the data from the infile
;
;	S. T. Megeath (4-7-00)
;
;	3/24/02 added capability to input IDL array or file
;
;-----------------------------------------------------------------

pro cluster_plot,rot,fov=fov,infile=infile,data=data

; set default fov
  if NOT keyword_set(fov) then fov="fov1"

; stop if neither infile or data keyword is set
  if (NOT keyword_set(infile) and NOT keyword_set(data)) then stop

; load in data if keyword set
  if keyword_set(data) then begin
	 coords = data
	 inc = n_elements(data[0,*])-1
  endif 

; load input file if keyword infile selected
  if keyword_set(infile) then begin
        coords = replicate(-1000.,2,20000)
	openr,1,infile
	i=-1
        while not eof(1) do begin
          readf,1,tmp
          if i lt 20000-1 then i = i + 1 else stop
	  coords[*,i] = tmp
        endwhile
	close,1
	inc = i
        coords = coords[*,0:inc]
   endif

; plot data
   for i =0,inc  do begin
     iracfov,coords[0,i],coords[1,i],rot,coords='celestial',/radec,fov=fov
   endfor

end
   
