;------------------------------------------------------------
;
;	iracfov
;
;	this program overlays the IRAC FOV onto images displayed
;	with disp_image.  It assumes that the x,y coordinates
;	are in RA,Dec with units of degrees.
;
;	x = RA in decimal degrees
;       y = Dec in decimal degrees
;       rot = position angle in degrees
;
;	S. T. Megadeath (4-5-00) 
;
;	last modified 4-11-00
;-------------------------------------------------------------

pro iracfov,x,y,posangle,radec=radec,coords=coords,fov=fov

  rot = posangle
  cos = 1.

  if NOT keyword_set(fov) then fov = "both"
  if keyword_set(radec) then cos = 1./cos(2.*3.14159264*y/360.)
  if NOT keyword_set(coords) then coords = 'spacecraft'

  casefov = string(fov)

; define rotation angle in radians and edges FOV of one array in 
; arcminutes
  rot = 2*3.14159264/360.*(rot)

if(coords eq 'spacecraft') then begin
; if coordinates are spacecraft
  x0 = -1*([2.56,-2.56,-2.56,2.56,2.56])
  y0 = [2.56,2.56,-2.56,-2.56,2.56]+3.25

  x1 = -1*([2.56,-2.56,-2.56,2.56,2.56])
  y1 = -1*([2.56,2.56,-2.56,-2.56,2.56])-3.25
endif else begin
; if coordinates are celestial
  x0 = -1*([2.56,-2.56,-2.56,2.56,2.56])
  y0 = [2.56,2.56,-2.56,-2.56,2.56]

  x1 = -1*([2.56,-2.56,-2.56,2.56,2.56])
  y1 = -1*([2.56,2.56,-2.56,-2.56,2.56])-6.5
endelse

; convert units into degress
  x0 = x0/60.
  y0 = y0/60.
  x1 = x1/60.
  y1 = y1/60.

; rotate
  x0rot = x0*cos(rot)+y0*sin(rot)
  y0rot = y0*cos(rot)-x0*sin(rot)

  x1rot = x1*cos(rot)+y1*sin(rot)
  y1rot = y1*cos(rot)-x1*sin(rot)

; FOVs

  case casefov of
    "fov1": begin
          oplot,cos*x0rot+x,y0rot+y,psym=0,color=201
         end
    "fov2": begin
          oplot,cos*x1rot+x,y1rot+y,psym=0,color=201
        end
    "both": begin
          oplot,cos*x0rot+x,y0rot+y,psym=0,color=201
          oplot,cos*x1rot+x,y1rot+y,psym=0,color=201
        end
   endcase


end
