;+
;cp_addmask.pro - makes an arbitrary polygon mask vector
;-

function cp_addmask,cpx,cpy,hdr

 xsz = size(cpx) & ysz = size(cpy)
 if xsz[0] gt 1 or ysz[0] gt 1 then begin 
  print,'ERROR in cp_addmask.pro: Invalid control points arrays'
  retall
 endif
 if xsz[xsz[0]+2] ne ysz[ysz[0]+2] then begin
  print,'ERROR in cp_addmask.pro: Need same number of values for X as Y'
  retall
 endif

 isz = [sxpar(hdr,'NAXIS'),sxpar(hdr,'NAXIS1'),sxpar(hdr,'NAXIS2')]

 if isz[0] ne 2 then begin
  print,'ERROR in cp_addmask.pro: 2D image header required!'
  retall
 endif

 nlines = xsz[xsz[0]+2]

 ptrs = ptrarr(nlines,/alloc) 

 xrng = [min(cpx),max(cpx)] & yrng = [min(cpy),max(cpy)]

 for i = 0,nlines-1 do begin
  sec = (i+1) mod nlines
  ny = abs(cpy[i] - cpy[sec])+1
  nx = abs(cpx[i] - cpx[sec])+1
  case 1 of
   nx eq 1 and ny eq 1: begin
    *ptrs[i] = lonarr(2,1)
    (*ptrs[i])[0,0] = [cpx[i],cpy[i]]
   end
   nx eq 1: begin
    *ptrs[i] = lonarr(2,ny)
    (*ptrs[i])[0,*] = cpx[i]
    (*ptrs[i])[1,*] = lindgen(ny) + min([cpy[i],cpy[sec]])
   end
   else: begin
    *ptrs[i] = lonarr(2,nx)
    (*ptrs[i])[0,*] = lindgen(nx) + min([cpx[i],cpx[sec]])
    (*ptrs[i])[1,*] = long(float(cpy[i]-cpy[sec])/float(cpx[i]-cpx[sec])*((*ptrs[i])[0,*]-cpx[i])+.5)+cpy[i]
   end
  endcase
 endfor
  
 nbad = 0
 num = lonarr(2,nlines)
 for j = xrng[0],xrng[1] do begin
  for i = 0,nlines-1 do begin
   tmp = where((*ptrs[i])[0,*] eq j,ntmp)
   if ntmp gt 0 then begin
    num[*,i] = [min((*ptrs[i])[1,tmp]),max((*ptrs[i])[1,tmp])] 
   endif else begin
    num[*,i] = [2l^31-1,-1]
   endelse
  endfor
  tru = [min(num[0,*]),max(num[1,*])]
  if nbad eq 0 then begin
   nbad = tru[1]-tru[0]+1
   ymask = (lindgen(nbad)+tru[0])*isz[2]+j
  endif else begin
   nbad = nbad + tru[1]-tru[0]+1
   ymask = [ymask, (lindgen(tru[1]-tru[0]+1)+tru[0])*isz[1]+j]
  endelse 
 endfor

;do it all again, rotated by 90 degrees...  :p
 for i = 0,nlines-1 do begin
  sec = (i+1) mod nlines
  ny = abs(cpy[i] - cpy[sec])+1
  nx = abs(cpx[i] - cpx[sec])+1
  case 1 of
   nx eq 1 and ny eq 1: begin
    *ptrs[i] = lonarr(2,1)
    (*ptrs[i])[0,0] = [cpx[i],cpy[i]]
   end
   ny eq 1: begin
    *ptrs[i] = lonarr(2,nx)
    (*ptrs[i])[0,*] = lindgen(nx) + min([cpx[i],cpx[sec]])
    (*ptrs[i])[1,*] = cpy[i]
   end
   else: begin
    *ptrs[i] = lonarr(2,ny)
    (*ptrs[i])[1,*] = lindgen(ny) + min([cpy[i],cpy[sec]])
    (*ptrs[i])[0,*] = long(float(cpx[i]-cpx[sec])/float(cpy[i]-cpy[sec])*((*ptrs[i])[1,*]-cpy[i])+.5)+cpx[i]
   end
  endcase
 endfor
  
 nbad = 0
 num = lonarr(2,nlines)
 for j = yrng[0],yrng[1] do begin
  for i = 0,nlines-1 do begin
   tmp = where((*ptrs[i])[1,*] eq j,ntmp)
   if ntmp gt 0 then begin
    num[*,i] = [min((*ptrs[i])[0,tmp]),max((*ptrs[i])[0,tmp])] 
   endif else begin
    num[*,i] = [2l^31-1,-1]
   endelse
  endfor
  tru = [min(num[0,*]),max(num[1,*])]
  if nbad eq 0 then begin
   nbad = tru[1]-tru[0]+1
   xmask = j*isz[2]+tru[0]+lindgen(nbad)
  endif else begin
   nbad = nbad + tru[1]-tru[0]+1
   xmask = [xmask, j*isz[1]+tru[0]+lindgen(tru[1]-tru[0]+1)]
  endelse 
 endfor

 return,merge_mask(xmask,ymask,/agree)

end
