;
; gather program
;
; The gather program takes 10 variables
;
; 1. infile is the name of the data file containing the photometry offsets (string)
; 2. nlines is the number of lines in the photometry file (int)
; 3. xsize is the number of gridpoints in the x direction (int)
; 4. ysize is the number of gridpoints in the y direction (int)
; 5. fitsfile is the name of the output fits file for the surface dentisty image (string)
; 6. xorigin is the x coordinate of the gridpoint at the origin in arcseconds(float)
; 7. xorigin is the x coordinate of the gridpoint at the origin in arcseconds(float)
; 8. gridsize is the spacing of the gridpoints in arcsec (float)
; 9. maglimit is the limiting faintest magnitude of stars to be included (float)
; 10. numstars is the number of stars taken into consideration at each point (int)
;-----------------------------------------------------------------
 
pro gather,infile,nlines,xsize,ysize,fitsfile=fitsfile,xorigin=xorigin,yorigin=yorigin,$
	gridsize=gridsize,maglimit=maglimit,numstars=numstars,image=image

  if NOT keyword_set(fitsfile) then fitsfile = "gather.fits"
  if NOT keyword_set(xorigin) then xorigin = 0
  if NOT keyword_set(yorigin) then yorigin = 0
  if NOT keyword_set(gridsize) then gridsize = 2.5
  if NOT keyword_set(maglimit) then maglimit = 25
  if NOT keyword_set(numstars) then numstars = 7

  data = fltarr(4,nlines)      ;This is the source#, raoffset, and decoffset in arcseconds, and mk
  image = fltarr(xsize,ysize)  ;This is the output density map

  openr,1,infile
  readf,1,data
  close,1

  q = findgen(numstars)            ;A useful index running from 0.0 to numstars-1.0
; w = 0.474833*exp(-(abs(q-((numstars-1)/2))))    ;This is a potential weighting function
  w = 1/(float(numstars))                 ;This is an even weighting function

  mags = data[3,*]
  data = data[*,where(mags LT maglimit)] ;This eliminates data for stars with magnitude > maglimit

  for i = 0,xsize-1 do begin
    for j = 0,ysize-1 do begin
      x = xorigin-gridsize*i  ;This is the RA offset of the gridpoint in question in Arcsec
      y = gridsize*j-yorigin  ;This is the DEC offset of the gridpoint in question in Arcsec
      dist = sqrt((data[1,*]-x)^2+(data[2,*]-y)^2) ;The distances of every star to the gridpoint in question
      dist = dist[sort(dist)]   ;Those distances sorted in ascending order
;      dist = dist[0:numstars]   ;The "numstars" smallest distances, still in ascending order

; peter's weigthing
;      image[i,j] = 1/(total((w*3.14159*(dist^2))/(q+0.5)))
; no weigthing
       image[i,j] = 1/(!pi*dist[numstars-1]^2)
    endfor
  endfor
  
  fits_write,fitsfile,image
; disp_image,image,findgen(xsize),findgen(ysize)
 
return
end
