;+
;
; rad_cmd1.pro - makes a radially partitioned set of CMDs from a PVDAT file.
;
; CALLING SEQUENCE:
;
;   rad_cmd1,in_dat,in_dist,radpix,maxdist=maxdist,center=center,myr=myr,useall=useall,$
;            title=title,xtitle=xtitle,ytitle=ytitle,charsize=charsize
;
;-

pro rad_cmd1,in_dat,in_dist,radpix,maxdist=maxdist,center=center,myr=myr,$
                 useall=useall,xrange=xrange,yrange=yrange,$
                 title=title,xtitle=xtitle,ytitle=ytitle,charsize=charsize,$
                 outdata=outdata,sat=sat,maglimit=maglimit

 if not keyword_set(myr) then myr = 1

 tmp_dat = in_dat

 if not keyword_set(center) then begin
  center = [median(in_dat[0,*]),median(in_dat[1,*])]
 endif

 if keyword_set(sat) then begin
  w = where(in_dat[2,*] gt sat[0] and in_dat[3,*] gt sat[1]); and in_dat[4,*] gt sat[2])
  in_dat = in_dat[*,w]
 endif
 if keyword_set(maglimit) then begin
  w = where(in_dat[2,*] lt maglimit[0] and in_dat[3,*] lt maglimit[1]); and in_dat[4,*] lt maglimit[2])
  in_dat = in_dat[*,w]
 endif

 dist = sqrt((in_dat[0,*]-center[0])^2 + (in_dat[1,*]-center[1])^2)

 if not keyword_set(maxdist) then begin
  if keyword_set(useall) then maxdist = max(dist)+1 else $
     maxdist = min([[max(in_dat[0,*]),max(in_dat[1,*])]-center,center])
 endif

 w_in = where(dist lt radpix)
 w_out = where(dist ge radpix and dist lt maxdist) 

 in_color = reform(in_dat[2,*]-in_dat[3,*])

 if not keyword_set(xrange) then xrange = [(min(in_color)-.5)>(-1),(max(in_color)+.5)<8]
 if not keyword_set(yrange) then yrange = [(max(in_dat[3,*])+.5)<25.,(min(in_dat[3,*])-.5)>0]

 old_pmulti = !p.multi
 !p.multi = [0,2,1]

 if not keyword_set(title) then usetitle = '' else usetitle = title+': '

 plot,in_color[w_in],in_dat[3,w_in],psym=7,symsize=.5,xrange=xrange,yrange=yrange,$
    /xstyle,/ystyle,$
    title=usetitle+'Source CMD',xtitle=xtitle,ytitle=ytitle,charsize=charsize
 hr_plot1,in_dist,myr=myr
 plot,in_color[w_out],in_dat[3,w_out],psym=7,symsize=.5,xrange=xrange,$
    yrange=yrange,/xstyle,/ystyle,$
    title=usetitle+'Background CMD',xtitle=xtitle,ytitle=ytitle,charsize=charsize
 hr_plot1,in_dist,myr=myr

 mo = moment(in_color[w_in],sdev=stdev)
 print,'Mean Source Color: '+strtrim(mo[0],2)+' +/- '+strtrim(stdev,2)
 mo = moment(in_color[w_out],sdev=stdev)
 print,'Mean Background Color: '+strtrim(mo[0],2)+' +/- '+strtrim(stdev,2)

 !p.multi = old_pmulti
 outdata = in_dat
 in_dat = tmp_dat

end
