;+
;   name:
;        skyflat.pro
;
;   purpose:
;        a procedure which takes the fits images in a given directory
;        (of sky flats, hopefully) and sorts them by filter and takes the
;	 darks and sorts them by channel, then median_stacks the darks 
;	 and subtracts them from the proper flat, then median_stacks the
;	 flats and saves the resulting final image for each filter. 
;
;   calling sequence:
;        skyflat,fdirname,ddirname,fwild=fwild,dwild=dwild
;
;   inputs:
;       fdirname = (string) subdirectory containing desired flats 
;       ddirname = (string) subdirectory containing desired darks 
;
;   keywords:
;       fwild = (string) set to directory AND wild characters if you
;               want to only take subset of flats in directory
;       dwild = (string) set to directory AND wild characters if you
;               want to only take subset of darks in directory
;
;   output:
;       five or six FITS files
;
;   called programs:
;	sf_dark.pro
;	sf_flat.pro
;   	rob_pf.pro
;
;   history:
;       written by R. Gutermuth - 5-23-01
;	last modified - 8-24-01
;
;-

pro skyflat,fdirname,ddirname,fwild=fwild,dwild=dwild

if not keyword_set(dwild) then dwild=0
if not keyword_set(fwild) then fwild=0

ddirname = rob_pf(/directory,title='Select a Directory with '+$
   'Dark Frames')

sf_dark,ddirname,wild=dwild

nddirname = filepath(root_dir=ddirname,subdir='sfdarks','')

fdirname = rob_pf(/directory,title='Select a Directory with '+$
   'Flat Fields')

sf_flat,fdirname,nddirname,wild=fwild

end

