
;****************************************************
;PhotVis shutdown routine
;****************************************************

pro photvis_shutdown,tlb_id

 widget_control,tlb_id,get_uvalue=state

;kill off any sub-windows
 if (*state).fcfg_id gt 0 then widget_control,(*state).fcfg_id,/destroy
 if (*state).pcfg_id gt 0 then widget_control,(*state).pcfg_id,/destroy
 if (*state).acfg_id gt 0 then widget_control,(*state).acfg_id,/destroy

;free the memory used
 ptr_free,(*state).main_dat, $
          (*state).conv_dat, $
          (*state).aper_dat, $
          (*state).find_dat, $
          (*state).find_status, $
          (*state).main_disp, $
          (*state).conv_disp, $
          state

;Kill the app!
 widget_control,tlb_id,/destroy

end

;****************************************************
;PhotVis photometry data save procedure
;****************************************************

pro photvis_save,tlb_id

 widget_control,tlb_id,get_uvalue=state
 if (*state).outnm eq '' then begin
  fname = dialog_pickfile(/write,path=(*state).path,dialog_parent=tlb_id,$
                          title='PhotVis: Save Photometry Data (IDL "SAVE") As')
 endif else fname = (*state).outnm

 if fname eq '' then return
 if file_test(fname) then begin
  huh = dialog_message(/question,'File Exists: Overwrite selected file?',$
                       dialog_parent=tlb_id)
  if huh eq 'No' then return
 endif
 widget_control,/hourglass

;now we use IDL "SAVE"
 w = where(*(*state).find_status le 0,nct)
 if nct eq 0 then begin
  doh = dialog_message('No valid detections! File not written!',$
                       dialog_parent=tlb_id)
  return
 endif
 pv_dat = fltarr(12,nct,/nozero)
 pv_dat[0:3,*] = (*(*state).find_dat)[*,w]
 pv_dat[4:10,*] = (*(*state).aper_dat)[*,w]
 pv_dat[11,*] = (*(*state).find_status)[w]
 save,filename=fname,pv_dat
 gd = dialog_message(/info,"To access pv_dat, type RESTORE,'"+fname+"'",$
                     dialog_parent=tlb_id)
 print,"RESTORE,'"+fname+"'"
 print,'[x,y,rnd,shp,aper,insky,outsky,mag,magerr,sky,skyerr,user_status]'
 widget_control,hourglass=0

end

;****************************************************
;PhotVis main display event handler
;****************************************************

pro photvis_draw_event,event

 widget_control,event.top,get_uvalue=state

 event.x = 0 > event.x < ((*state).port_size-1)
 event.y = 0 > event.y < ((*state).port_size-1)

 off_ev = fix([event.x,event.y] / (*state).zoom_level) + (*state).view_offset
 if off_ev[0] ge 0 and off_ev[0] le (*(*state).main_dat[3])[1]-1 and $
    off_ev[1] ge 0 and off_ev[1] le (*(*state).main_dat[3])[2]-1 then inbnds=1 else inbnds=0

 case event.type of
  2: begin  ;motion events first, as they are the ones most often generated

;update the labels
   widget_control,(*state).label[0],set_value=strtrim(off_ev[0],2)
   widget_control,(*state).label[1],set_value=strtrim(off_ev[1],2)

   if inbnds then begin
    widget_control,(*state).label[2],set_value= $
       strtrim((*(*state).main_dat[1])[off_ev[0],off_ev[1]],2)
    if (*state).find_state[0] then widget_control,(*state).label[3], $
       set_value=strtrim((*(*state).conv_dat[1])[off_ev[0],off_ev[1]],2)
   endif else begin
    widget_control,(*state).label[2],set_value=strtrim(-1.,2)
    widget_control,(*state).label[3],set_value=strtrim(-1.,2)
   endelse

;if a button is currently being held, run pv_imexam
   switch (*state).but_hold of
    0: break
    4: pv_place_detect,event
    1: begin
     pv_imexam,event
     if (*state).but_hold eq 1 then begin
      widget_control,(*state).label[8],set_value = ' - '
      widget_control,(*state).label[9],set_value = ' - '
     endif
    end

    else: break
   endswitch
  end

  0: begin  ;button click events
   switch event.press of
    4: pv_place_detect,event ;redefine x and y of event to be nearest detection
    1: begin
     case event.clicks of
      2: begin
       if event.press eq 1 then pv_add_object,event
       return
      end
      1: begin
       if event.press eq 1 then begin
        if (*state).active_obj ne -1 then begin
         (*state).active_obj = -1
         pv_photdisp,event
        endif
        widget_control,(*state).label[8],set_value = ' - '
        widget_control,(*state).label[9],set_value = ' - '
       endif
       pv_imexam,event
      end
      else:break
     endcase
    end
    else: break
   endswitch
   (*state).but_hold = event.press
  end

  1: begin  ;button release events
   if (*state).but_hold eq 4 then pv_place_detect,event
   (*state).but_hold = 0
  end

  else: break  ;don't care about anything else yet
 endcase

end


;****************************************************
;PhotVis main display event handler for multi-object selection
;****************************************************


pro photvis_select_event,event

 widget_control,event.top,get_uvalue=state

 event.x = 0 > event.x < ((*state).port_size-1)
 event.y = 0 > event.y < ((*state).port_size-1)

 off_ev = fix([event.x,event.y] / (*state).zoom_level) + (*state).view_offset
 if off_ev[0] ge 0 and off_ev[0] le (*(*state).main_dat[3])[1]-1 and $
    off_ev[1] ge 0 and off_ev[1] le (*(*state).main_dat[3])[2]-1 then inbnds=1 else inbnds=0

; if (*state).active_obj ne -1 then begin
;  (*state).active_obj = -1
;  pv_photdisp,event
; endif

 case event.type of
  2: begin  ;motion events first, as they are the ones most often generated

;update the labels

   widget_control,(*state).label[0],set_value=strtrim(off_ev[0],2)
   widget_control,(*state).label[1],set_value=strtrim(off_ev[1],2)

   if inbnds then begin
    widget_control,(*state).label[2],set_value= $
       strtrim((*(*state).main_dat[1])[off_ev[0],off_ev[1]],2)
    if (*state).find_state[0] then widget_control,(*state).label[3],$
       set_value=strtrim((*(*state).conv_dat[1])[off_ev[0],off_ev[1]],2)
   endif else begin
    widget_control,(*state).label[2],set_value=string(-1.)
    widget_control,(*state).label[3],set_value=string(-1.)
   endelse

;if a button is currently being held, draw a box
   if (*state).but_hold gt 0 then begin
    wset,(*state).win_id
    device,copy=[0,0,(*state).port_size,(*state).port_size,0,0,(*state).clip_id]
    (*state).select[2:3] = [event.x,event.y]
    box = (*state).select
    box_plot,box[0],box[2],box[1],box[3]
   endif
  end

  0: begin  ;button click events
   (*state).select[0:1] = [event.x,event.y]
   (*state).but_hold = event.press
  end

  1: begin  ;button release events
   wset,(*state).win_id
   device,copy=[0,0,(*state).port_size,(*state).port_size,0,0,(*state).clip_id]
   (*state).select[2:3] = [event.x,event.y]
   box = (*state).select
   box_plot,box[0],box[2],box[1],box[3]
   rbox = box
   rbox[0:1] = fix(box[0:1] / (*state).zoom_level + (*state).view_offset)
   rbox[2:3] = fix(box[2:3] / (*state).zoom_level + (*state).view_offset)
   if (*state).n_objects[1] gt 0 then begin
      w = where(((*(*state).find_dat)[0,*] lt max(rbox[[0,2]]) and $
              (*(*state).find_dat)[0,*] gt min(rbox[[0,2]])) and $
             ((*(*state).find_dat)[1,*] lt max(rbox[[1,3]]) and $
              (*(*state).find_dat)[1,*] gt min(rbox[[1,3]])),wct)
   endif else wct = 0
   if wct gt 0 then begin
    case (*state).but_hold of
     4: begin
      huh = dialog_message(/ques,'Reject All Selected Objects?')
      flg = 9
     end
     1: begin
      huh = dialog_message(/ques,'Include All Selected Objects?')
      flg = -1
     end
     else: huh = 'No'
    endcase

    if huh eq 'Yes' then begin
     case flg of
      9: begin
       uw = where((*(*state).find_status)[w] eq -2,uct)
       if uct gt 0 then begin
        for i = 0l,uct-1 do begin
         ind = lindgen((*state).n_objects[1])
         kw = where(ind ne w[uw[i]],kct)
         if kct gt 0 then begin
          *(*state).find_dat = temporary((*(*state).find_dat)[*,kw])
          *(*state).find_status = temporary((*(*state).find_status)[kw])
          *(*state).aper_dat = temporary((*(*state).aper_dat)[*,kw])
          (*state).n_objects[1] = kct
         endif
        endfor
       endif
       uw = where((*(*state).find_status)[w] ne -2,uct)
       if uct gt 0 then (*(*state).find_status)[w[uw]] = flg
      end
      -1: begin
       uw = where((*(*state).find_status)[w] gt 0,uct)
       if uct gt 0 then (*(*state).find_status)[w[uw]] = flg
      end
      else: break
     endcase

     if (*state).find_state[0] eq 0 then begin
      w = where(*(*state).find_status lt 0,nct)
     endif else w = where(*(*state).find_status le 0,nct)
     (*state).n_objects[0] = nct
     widget_control,(*state).label[4],set_value=strtrim((*state).n_objects[0],2)
    endif
   endif

   pv_redisplay,event.top
   (*state).but_hold = 0
  end

  else: break  ;don't care about anything else yet
 endcase

end


;****************************************************
;PhotVis pan field event handler
;****************************************************

pro photvis_field_event,event

 widget_control,event.top,get_uvalue=state

 box_size = fix((*state).view_size * (*state).field_scl+.5)

;keep selection box in bounds
 event.x = event.x > 0 < ((*state).field_size[0] - 1)
 event.y = event.y > 0 < ((*state).field_size[1] - 1)

 field_off = [event.x,event.y] - box_size/2 + 1

 case event.type of
  0: (*state).but_hold = event.press  ;button press events
  1: (*state).but_hold = 0            ;button release events
  else: break                         ;don't care about anything else yet
 endcase

 if (*state).but_hold gt 0 or event.type eq 1 then begin
  wset,(*state).field_id
  device,copy=[0,0,(*state).field_size[0],(*state).field_size[1],$
               0,0,(*state).field_pix_id]
  box_plot,field_off[0],field_off[0]+box_size-1,field_off[1],$
           field_off[1]+box_size-1

  if event.type eq 1 then begin

;set offset, and make sure it's in bounds
   (*state).view_offset[0] = fix(field_off[0]/(*state).field_scl+.5)
   (*state).view_offset[1] = fix(field_off[1]/(*state).field_scl+.5)

;now set viewport image for both main and conv
   pv_dispcalc,state

;update the display!
   pv_redisplay,event.top
  endif
 endif

end

;****************************************************
;PhotVis zoom button event handler
;****************************************************

pro photvis_zoom_event,event

 widget_control,event.top,get_uvalue=state
 widget_control,/hourglass

 case event.value of
  'Zoom In': zoomchg = 1                   ;Zoom in 2x

  'Zoom Out': zoomchg = -1                 ;Zoom out 2x

  'Zoom Home': default = 1                 ;Back to pix-for-pix scale

  'Recenter': center = 1                   ;Recenter viewport

  else: huh = dialog_message(/warning,$
           'No valid zoom buttons were pushed?!  How did that happen?!')
 endcase

 pv_zoomcalc,state,zoomchg,default=default,center=center

 pv_dispcalc,state

;redisplay and kill the hourglass
 pv_redisplay,event.top
 widget_control,hourglass=0

end

;****************************************************
;PhotVis mode selection event handler
;****************************************************

pro photvis_mode_event,event

 widget_control,event.top,get_uvalue=state
 widget_control,/hourglass

 case event.value of

  'Main Image': begin  ;back to main image display

;set current display and data pointers to the main stuff
   (*state).curr_disp = (*state).main_disp
   (*state).curr_dat = (*state).main_dat
  end

  'Convolved Image': begin  ;convolved image display

;set current display and data pointers to the convolved stuff
   (*state).curr_disp = (*state).conv_disp
   (*state).curr_dat = (*state).conv_dat
  end

  else: huh = dialog_message(/warning,$
                 'No valid modes were selected?!  How did that happen?!')
 endcase

 pv_redisplay,event.top
 scl = *(*state).curr_disp[1]
 for i = 0,1 do widget_control,(*state).scale[i],set_value=scl[i]
 widget_control,hourglass=0

end

;****************************************************
;PhotVis event handler for all the reject compound widgets...  >:-(
;****************************************************

pro photvis_event,event

 widget_control,event.top,get_uvalue=state

;stupid scaling fields are the same, thus switch instead of case
 switch event.id of
  (*state).scale[0]: begin
  end
  (*state).scale[1]: begin
   scl = fltarr(2,/nozero)
   for i = 0,1 do begin
    widget_control,(*state).scale[i],get_value=temp
    scl[i] = temp
   endfor

;if min and max are ok, rescale current...  otherwise, reset fields
   if scl[0] lt scl[1] then begin
    *(*state).curr_disp[1] = scl
    *(*state).curr_dat[5] = bytscl(*(*state).curr_dat[1],min=scl[0],max=scl[1])
    pv_dispcalc,state
    pv_redisplay,event.top
   endif else for i = 0,1 do widget_control,(*state).scale[i],set_value=$
                                            (*(*state).curr_disp[1])[i]
   break
  end

;aperture fields...
  (*state).ap_fields[0]: begin
   minval = 1
   maxval = (*(*state).aper_dat)[1,(*state).active_obj]
   field = 0
  end

  (*state).ap_fields[1]: begin
;looks redundant, but this is switch, not case!!
;saves a lot of redundant code in the long run...
   if event.id eq (*state).ap_fields[1] then begin
    minval = (*(*state).aper_dat)[0,(*state).active_obj]
    maxval = ((*(*state).aper_dat)[2,(*state).active_obj]-1)
    field = 1
   endif
  end

  (*state).ap_fields[2]: begin
   if event.id eq (*state).ap_fields[2] then begin
    minval = ((*(*state).aper_dat)[1,(*state).active_obj]+1)
    maxval = 73.
    field = 2
   endif
  end

;aperture reset button
  (*state).ap_fields[3]: begin
   obj = (*state).active_obj
   if event.id eq (*state).ap_fields[3] then begin
    if obj ge 0 then begin
     for i = 0,2 do begin
      widget_control,(*state).ap_fields[i],set_value=(*state).aper_defaults[i]
      (*(*state).aper_dat)[i,obj] = (*state).aper_defaults[i]
     endfor
    endif
   endif else begin
    widget_control,event.id,get_value=temp
    newtemp = temp > minval < maxval
    if newtemp ne temp then widget_control,event.id,set_value=newtemp
    (*(*state).aper_dat)[field,obj] = newtemp
   endelse
   if obj ge 0 then begin
    aper,*(*state).main_dat[1],(*(*state).find_dat)[0,obj],$
         (*(*state).find_dat)[1,obj],mag,errap,sky,skyerr,(*state).aper_defaults[3], $
         (*(*state).aper_dat)[0,obj],(*(*state).aper_dat)[1:2,obj],[0,0],/silent,/exact
    (*(*state).aper_dat)[3:6,obj] = [mag+((*state).aper_defaults[4]-25),errap,sky,skyerr]
   endif
   pv_photdisp,event
   break
  end

  (*state).dmode: begin
   case event.index of
    0: widget_control,(*state).draw,event_pro='photvis_draw_event'
    1: begin
     (*state).active_obj = -1
     f_event = {top:(*state).tlb,x:256l,y:256l}
     pv_photdisp,f_event
     widget_control,(*state).draw,event_pro='photvis_select_event'
    end
    else: break
   endcase
   break
  end

;top menu events...
  (*state).top_menu: begin
   case event.value of

    'New PhotVis Session': photvis;,(*state).path

    'Open FITS Image': begin

;get filename from user
     *(*state).main_dat[0] = dialog_pickfile(/must_exist,get_path=newpath,$
        path=(*state).path,title='PhotVis: Select FITS Image')
     if *(*state).main_dat[0] ne '' then begin
      (*state).path = newpath
      pv_fitsread,state
      if (*state).find_state[0] eq 1 then pv_find_init,state
      pv_dispcalc,state
      pv_redisplay,(*state).tlb  ;display everything
      pv_sense,state
      for i=0,1 do widget_control,(*state).scale[i],set_value=(*(*state).curr_disp[1])[i]
     endif
    end

    'Save Current Session': pv_save_session,state

    'Change IDL Working Directory': rob_cd

    'Restore Saved Session': pv_restore_session,state

    'Save Photometry Results': begin
     if (*state).n_objects[0] gt 0 then photvis_save,event.top else $
        huh = dialog_message('PhotVis: No valid objects to save!')
    end

    'Quit Current Session': widget_control,event.top,/destroy

    'FITS Header': begin
     if n_elements(*(*state).main_dat[2]) gt 0 then begin
      xdisplayfile,'',text=*(*state).main_dat[2],group=(*state).tlb,$
                   title='PhotVis: FITS Header: '+*(*state).main_dat[0]
     endif else begin
      huh = dialog_message('PhotVis: No header to display!!')
     endelse
    end

    'Statistics': begin
     stats = pv_stats_text(state)

     if n_elements(stats) gt 1 then xdisplayfile,'',text=stats,group=$
        (*state).tlb,title='PhotVis: Statistics for '+*(*state).main_dat[0]$
        else huh = dialog_message('PhotVis: No statistics to display!!')   

    end

    'Object Detection': begin
     if (*state).fcfg_id[0] gt 0 then begin
      widget_control,(*state).fcfg_id,map=1,show=1
     endif else pv_find_config,event.top
    end

    'Aperture Photometry': begin
     if (*state).acfg_id[0] gt 0 then begin
      widget_control,(*state).acfg_id,map=1,show=1
     endif else pv_phot_config,event.top
    end

    'Overplots': begin
     if (*state).pcfg_id[0] gt 0 then begin
      widget_control,(*state).pcfg_id,map=1,show=1
     endif else pv_plot_config,event.top
    end

    'PhotVis Help': huh = dialog_message(/info,dialog_parent=event.top, $
       "Email questions, bug reports, etc. to rguter@astro.pas.rochester.edu")
    else: break
   endcase
  end
  else: break
 endswitch

end

;****************************************************
;PhotVis main initialization procedure
;****************************************************

pro photvis,block=block;,path,inname=inname,outname=outname,default=default

 if not keyword_set(block) then no_block = 1

 state = pv_state_init()

 pv_gui_init,state

;hand off to xmanager
 xmanager,'photvis',(*state).tlb,no_block=no_block,cleanup='photvis_shutdown'

end
