;----------------------------------------------------------------------------
;
;   imexam
;
;   This program examines an image using a radial plot, histogram,
;       and a display of pixel values.  The image must already be
;       displayed.
;
;   RAG-Just makes one set of plots.
;
;-----------------------------------------------------------------------------


pro pv_imexam,draw_event

 widget_control,draw_event.top,get_uvalue=state

 box_size = fix((*state).plot_params[2]/2)
 p_box_size = box_size*(*state).zoom_level

;x and y_mos are for scaling axes to actually have real coords
;currently just unzoomed pix coords
 x_mos = findgen((*state).port_size)/(*state).zoom_level+$
         (*state).view_offset[0]
 y_mos = findgen((*state).port_size)/(*state).zoom_level+$
         (*state).view_offset[1]

;actual pix coords of event
 if (*state).active_obj ne -1 then begin
  xcoord = fix((*(*state).find_dat)[0,(*state).active_obj]+.5)
  ycoord = fix((*(*state).find_dat)[1,(*state).active_obj]+.5)
 endif else begin
  xcoord = fix(draw_event.x / (*state).zoom_level + (*state).view_offset[0])
  ycoord = fix(draw_event.y / (*state).zoom_level + (*state).view_offset[1])
 endelse

 xcoord = xcoord > box_size < ((*(*state).main_dat[3])[1] - 1 - box_size)
 ycoord = ycoord > box_size < ((*(*state).main_dat[3])[2] - 1 - box_size)

 tempx = fix((xcoord  - (*state).view_offset[0]) * (*state).zoom_level)
 tempy = fix((ycoord  - (*state).view_offset[1]) * (*state).zoom_level)

;save old p.mulit
 oldpmulti = !p.multi
 !p.multi = [0,2,2]
 wset,(*state).win_id

;coords of image overplot box
 x1 = (tempx - p_box_size); > 0
 x2 = (tempx + p_box_size); < ((*state).port_size - 1)
 y1 = (tempy - p_box_size); > 0
 y2 = (tempy + p_box_size); < ((*state).port_size - 1)

;plot square
 wset,(*state).win_id
 device,copy=[0,0,(*state).port_size,(*state).port_size,0,0,(*state).clip_id]
 box_plot,x1,x2,y1,y2,/device

;pix coords to get data from images
 x1 = (xcoord - box_size)
 x2 = (xcoord + box_size)
 y1 = (ycoord - box_size)
 y2 = (ycoord + box_size)

 wset,(*state).plot_id

;add local stats update
 temp = ptr_new((*(*state).main_dat[1])[x1:x2,y1:y2])
 out = pv_get_stats(temp)
 ptr_free,temp
 mnmxbox = [min([reform((*(*state).main_dat[1])[xcoord,y1:y2]),$
               reform((*(*state).main_dat[1])[x1:x2,ycoord])]),$
            max([reform((*(*state).main_dat[1])[xcoord,y1:y2]),$
               reform((*(*state).main_dat[1])[x1:x2,ycoord])])]
 widget_control,(*state).label[5],set_value = strtrim(out[0],2)
 widget_control,(*state).label[6],set_value = strtrim(out[1],2)
 widget_control,(*state).label[7],set_value = strtrim(out[2],2)
 widget_control,(*state).label[10],set_value = strtrim(mnmxbox[0],2)
 widget_control,(*state).label[11],set_value = strtrim(mnmxbox[1],2)

;slice plots
 plot,(*(*state).main_dat[1])[xcoord,y1:y2],/ynozero,psym=10,$
      title='Column: '+strtrim(xcoord,2),yrange=mnmxbox,xstyle=1
 plot,(*(*state).main_dat[1])[x1:x2,ycoord],/ynozero,psym=10,$
      title='Row: '+strtrim(ycoord,2),yrange=mnmxbox,xstyle=1

;surface plot
 surface,(*(*state).main_dat[1])[x1:x2,y1:y2]>mnmxbox[0]<mnmxbox[1],$
         title='Surface Plot',zrange=mnmxbox,xstyle=1,ystyle=1

;radial profile plot
 rad = fltarr(2*box_size+1,2*box_size+1)
 for i = -box_size, box_size do begin
  for j = -box_size, box_size do begin
   rad[i+box_size,j+box_size] = sqrt(i^2+j^2)
  endfor
 endfor
 plot,reform(rad),reform((*(*state).main_dat[1])[x1:x2,y1:y2])$
      >mnmxbox[0]<mnmxbox[1],$
      /ynozero,psym=3,title='Radial Plot',yrange=mnmxbox

 wset,(*state).win_id

 !p.multi=oldpmulti

end
