;+
;   NAME:
;     FDS_MASTER
;
;   PURPOSE:
;     This function (not really meant to be used on it's own) checks input for 
;     validity in the specified log mode, changes input if not valid, and either
;     returns a one if the end state of input valid or returns a zero otherwise.
;
;   CALLING SEQUENCE:
;     result = logcheck(input)
;
;   INPUTS:
;     input = can be anything, including unspecified
;
;   KEYWORDS:
;       SAVE = set to make logcheck use file saving method (default is to use 
;              file reading method)
;       NO_SAVE = set to jump out of logcheck without doing anything
;
;   OUTPUTS:
;       result is a boolean integer
;
;   SIDE EFFECTS:
;       input can and frequently is modified by this function
;
;   CALLED PROCEDURES:
;       No non-standard subroutines are called
;
;   MODIFICATION HISTORY:
;     5-15-02: completed and integrated into pipeline (RAG)
;
;-


function logcheck,input,save=save,no_save=no_save

;if no_save is set, don't do anything, just get out
 if keyword_set(no_save) then return,0
 if keyword_set(save) and n_elements(input) eq 0 then input=1

 su = size(input)
 neednm = 0
 result = 1

;if input exists, isn't a string, and is a zero, get out
 if su[1] gt 0 then begin
  if su[su[0]+1] ne 7 then begin
   if input eq 0 then return,0

;ok, we need a gui to get a filename
   neednm = 1

;if input's a string, not an existing file, and we're in read mode,
;whine and get the filename gui
  endif else begin
   if not file_test(input) then begin
    if not keyword_set(save) then begin
     print,'ERROR in logcheck.pro: '+input+' does not exist!'
     neednm = 1
    endif

;if input is an existing file and we're in write mode, whine and
;ask if we need the filename gui with a question gui.
   endif else begin
    if keyword_set(save) then begin
     huh = dialog_message(/question,'File '+input+' exists...  '+$
        'Overwrite?')
     if huh eq 'No' then needmn = 1
    endif
   endelse
  endelse

;if we need the filename gui and we're in save mode, do it with
;a while but allow cancels
  if neednm then begin
   if keyword_set(save) then begin
    huh = 'No'
    while huh eq 'No' do begin
     input = dialog_pickfile(title='Save logfile as')
     if file_test(input) then begin
      huh = dialog_message(/question,'File '+input+' exists...'+$
         '  Overwrite?')
     endif else huh = 'Yes'
    endwhile
    text = ['Saving logfile to ','saving...']

;if we're in read mode, do one gui, and file must exist
   endif else begin
    input = dialog_pickfile(/must_exist,title=$
     'Pick logfile to (re)run')
    text = ['Restoring logfile ','restoration...']
   endelse

;if not a cancel, great, but otherwise, return a zero
   if input ne '' then print,text[0]+input else begin
    result = 0
    print,'Cancelling logfile '+text[1]
   endelse
  endif
 endif else result = 0

 return,result

end

