;..............................................................................
;
;   This program will generate a header for a fits cube using the x, y vectors
;
;   parameters  newheader - name of header variable
;		x         - x vector
;		y         - y vector
;		bitpix    - set to number bits per pixel
;	
;   keywords	cosy	  - set to take out cos(dec) term from x vector.
;                           (multiply cdelt1 by cos(median(y))
;	      
;
; 98 S. T. Megeath
; 7-27-2001 - modified from l,b to ra-dec (S. T. Megeath)
;..............................................................................

pro makeheader2d_radec_vector,newheader,x,y,bitpix,cosy=cosy



naxis1 = n_elements(x)
naxis2 = n_elements(y)
crpix1 = fix(naxis1/2)
crpix2 = fix(naxis2/2)
crval1 = x[crpix1-1]
crval2 = y[crpix2-1]
cdelt1 = x[1]-x[0]
cdelt2 = y[1]-y[0]

if keyword_set(cosy) then cdelt1 = cdelt1*cos(2.*!pi/360.*crval2)

newheader = "SIMPLE  =                    T /image conforms to FITS standard                 "
;newheader = [newheader,"BITPIX  =                   16 /bits per data value                             "]
newheader = [newheader,string(format = $
'("BITPIX  = ",i20," / bits per data value                               ")',crval1)]
newheader = [newheader,"NAXIS   =                    2 /number of axes                                  "]
newheader = [newheader,string(format = $
'("NAXIS1  = ",i20," /                                                   ")',naxis1)]
newheader = [newheader,string(format = $
'("NAXIS2  = ",i20," /                                                   ")',naxis2)]
newheader = [newheader,"EXTEND   =                   T /file may contain extensions                     "]
newheader = [newheader,"BSCALE  =              1.00000 /  REAL = TAPE*BSCALE + BZERO                    "]
newheader = [newheader,"BZERO   =              0.00000 /                                                "]
newheader = [newheader,"BUNIT   = '                  ' /  UNIT OF TOTAL INTENSITY                       "]
newheader = [newheader,"BLANK   =               -1000  / BLANK VALUE                                    "]
newheader = [newheader,"OBJECT  = '                  ' /  SOURCE NAME                                   "]
newheader = [newheader,string(format = $
'("CRVAL1  = ",f20.12," / REF VALUE POINT DEGREE                         ")',crval1)]
newheader = [newheader,string(format = $
'("CRPIX1  = ",f20.12," / REF POINT PIXEL LOCATION                       ")',crpix1)]
newheader = [newheader,"CTYPE1  = 'RA---SIN          ' / COORD TYPE : VALUE IS DEGR                     "]
newheader = [newheader,string(format = $
'("CDELT1  = ",f20.12," / COORD VALUE INCREMENT WITH COUNT DGR           ")',cdelt1)]
newheader = [newheader,"CROTA1  =       0.000000000000 / NO ROTATION                                    "]
newheader = [newheader,string(format = $
'("CRVAL2  = ",f20.12," / REF VALUE POINT DEGREE                         ")',crval2)]
newheader = [newheader,string(format = $
'("CRPIX2  = ",f20.12," / REF POINT PIXEL LOCATION                       ")',crpix2)]
newheader = [newheader,"CTYPE2  = 'DEC--SIN          ' / COORD TYPE : VALUE IS DEGR                     "]
newheader = [newheader,string(format = $
'("CDELT2  = ",f20.12," / COORD VALUE INCREMENT WITH COUNT DGR           ")',cdelt2)]
newheader = [newheader,"CROTA2  =       0.000000000000 / NO ROTATION                                    "]
newheader = [newheader,"EQUINOX =       2000           / NO ROTATION                                    "]

newheader = [newheader,"TELESCOP= '                  ' /                                                "]
newheader = [newheader,"END     "]
end
