;revised from Rob's irac_mosaic
;to work with c2d's naming convention and
;automatically set nosip keyword
;
;	T. Huard  07-15-2004
;	Last Revised  07-15-2004   5:00PM


; Use WCS info to mosaic IRAC data

pro irac_mosaic_c2d,infiles=infiles,nmos=nmos,nodistcorr=nodistcorr,nocrcorr=nocrcorr,derotate=derotate,channels=channels,nohdr=nohdr,chopsig=chopsig,workdir=workdir

 nosip=1 ; automatically set this original keyword (i.e., input must be BCD/ICD)
 
 if keyword_set(channels) then begin
  channels = fix(channels[uniq(channels,sort(channels))])
  w = where(channels le 4 or channels ge 1,wgd)
  if wgd gt 0 then channels = channels[w] else begin
   print,'ERROR: Invalid channels specified'
   retall
  endelse
 endif else channels = [1,2,3,4]
  
 if keyword_set(nosip) then sipstr='SPITZER_I*_*cd.fits' else sipstr='f*.fits'
 if not keyword_set(nmos) then nmos=4 else nmos = fix(nmos)
 if strlowcase(!version.os_family) eq 'windows' then sls = '\' else sls = '/'

 if not keyword_set(infiles) then begin
  indir = dialog_pickfile(/directory,title='Select Input Images Directory')
  ftot = findfile(indir+sipstr,count=nft)
  if nft eq 0 then begin
   print,'ERROR: No valid files found!'
   retall
  endif
 endif else begin
  ftot = infiles
  nft = n_elements(ftot)
 endelse

 if not keyword_set(workdir) then workdir = dialog_pickfile(/directory,title='Select Working Directory')
 if workdir ne '' then begin
  if not file_test(workdir) then begin
   file_mkdir,workdir
  endif
  file_mkdir,workdir+'temp'
 endif else begin
  print,'Cancelled...'
  retall
 endelse

 nchan = n_elements(channels)
 nfc = nft/nchan
 ftot = reform(ftot,nfc,nchan)
 f = ftot[0:nfc-1,channels[0]-1]
 irac_sort_data,f,xoffsets,yoffsets,workdir,nmos,derotate=derotate
 nf = n_elements(f)

 if nf eq nfc or keyword_set(nohdr) then begin
  hdr_mode = 0
  outfiles = workdir+'ch'+strtrim(channels,2)+'_mosaic.fits'
 endif else begin
  hdr_mode = 1
  outfiles = strarr(2,nchan)
  for i=0,nchan-1 do outfiles[*,i] = workdir+'ch'+strtrim(channels[i],2)+'_hdr_'+['long','short']+'.fits'
  outfiles = reform(outfiles,2*nchan)
 endelse

 for c = 0,nchan*(1+hdr_mode)-1 do begin
  
  hdr_type = (c+1) mod 2
  if hdr_mode eq 0 then hdr_type = 0

  chan = channels[c/(1+hdr_mode)]
  f = ftot[hdr_type:nfc-1:(1+hdr_mode),chan-1]

;get and sort filenames and WCS offsets and get make working directory
 if hdr_mode eq 1 and hdr_type eq 0 then begin
  sdx = xoffsets
  sdy = yoffsets
  sdorder = order
 endif

 irac_sort_data,f,xoffsets,yoffsets,workdir,nmos,derotate=derotate,order=order

 nf = n_elements(f)
 npm = nf/nmos

 if hdr_mode eq 1 and hdr_type eq 0 then begin
  xoffsets = sdx
  yoffsets = sdy
  f = (reform(ftot[hdr_type:nfc-1:(1+hdr_mode),chan-1],nmos,npm))[sdorder]
 endif

 f2 = f 

;arrange temp filenames, get local offsets, and then attempt cosmic ray cleanup

 if not keyword_set(nocrcorr) then begin
  for j = 0,npm-1 do begin
   for i = 0,nmos-1 do begin
    tmp = strsplit(f[i,j],sls,/extract)
    f2[i,j] = workdir+'temp'+sls+tmp[n_elements(tmp)-1]
    if i eq 0 then begin
     fits_read,f[i,j],im,hdr,/header_only
     mhdr = hdr; & xoff = fltarr(nmos) & yoff = xoff
     if j eq 0 then getrot,mhdr,rot1
    endif
;     crpix = sxpar(mhdr,'CRPIX*') - 1
;    endif else begin
;     xyxy,hdr,mhdr,crpix[0],crpix[1],xtmp,ytmp
;     xoff[i] = xtmp-crpix[0] & yoff[i] = ytmp-crpix[1]
;    endelse
   endfor

;time to play with the new toy
   wfun = where_overlap(f,xoffsets,yoffsets,j,nfun)
   if nfun eq 0 then begin
    wfun2 = lindgen(nmos)+j*nmos
    delvarx,wfun
   endif else begin
    wfun2 = [lindgen(nmos)+j*nmos,wfun]
    wfun = lindgen(nfun)+nmos
   endelse

   xoff = xoffsets[wfun2] - round(xoffsets[wfun2[0]])
   yoff = yoffsets[wfun2] - round(yoffsets[wfun2[0]])

   if keyword_set(nosip) then begin
;    gain = sxpar(hdr,'EXPTIME')/sxpar(hdr,'FLUXCONV')*sxpar(hdr,'GAIN')
    gain = sxpar(hdr,'GAIN')
   endif else begin
    gain = sxpar(hdr,'SIPCGAIN')
;   gain = [3.65,3.71,3.55,3.85]
   endelse

   cosmic_clean,f[wfun2],xoff,yoff,f2[*,j],sec_overlap=wfun,gain=gain,chopsig=chopsig,expsz=70.,nodistcorr=nodistcorr;[chan-1]
;   irac_cosmic_clean,f[*,j],xoff,yoff,f2[*,j],chopsig=chopsig

  endfor
 endif else begin
  for j = 0,npm-1 do begin
   for i = 0,nmos-1 do begin
    tmp = strsplit(f[i,j],sls,/extract)
    f2[i,j] = workdir+'temp'+sls+tmp[n_elements(tmp)-1]
    fits_datmask,f[i,j],im,hdr,mask
    fits_datmask,/write,f2[i,j],im,hdr,mask
   endfor
  endfor
 endelse

; stop

;work with temp files from now on, save orig filenames
 tmp = f & f = f2 & f2 = tmp


;----------------------------------

; if keyword_set(refine) and hdr_mode eq hdr_type then begin

; for j=0,npm-1 do begin

;read in master
;  fits_datmask,f[0,j],im,hdr,tempmask

;  if tempmask[0] eq -1 then begin
;   mask = finite(im)
;   w=where(mask eq 1,ngd)
;   if ngd gt 0 then w2 = where(im[w] lt 0,nbd)
;   if nbd gt 0 then mask[w[w2]] = 0
;   tempmask = where(mask eq 0,nbd)
;  endif else begin
;   mask = im*0.+1.
;   mask[tempmask]=0.
;   nbd = n_elements(tempmask)
;  endelse

;  med = median(im)
;  if nbd gt 0 then im[tempmask] = 0

;  irac_expand,im,hdr,mask,70
;  kx = fltarr(3,3) & ky = fltarr(3,3)
;  kx[0,1] = 1. & ky[1,0] = 1.
;
;  if not keyword_set(nodistcorr) then irac_dist,im,hdr,mask,kx,ky,70
;
;  if keyword_set(derotate) then irac_rot,im,hdr,mask,kx,ky,rot1
;  
;  master = poly_2d(temporary(im),kx,ky,1,missing=0)
;  mmask = poly_2d(temporary(mask),kx,ky,1,missing=0)
;  delvarx,kx,ky
;  mhdr = hdr

;refine master's submosaic offsets using cross-correllation
;  for i=1l,nmos-1 do begin

;   fits_datmask,f[i,j],im,hdr,tempmask 
;
;   if tempmask[0] eq -1 then begin
;    mask = finite(im)
;    w=where(mask eq 1,ngd)
;    if ngd gt 0 then w2 = where(im[w] lt 0,nbd)
;    if nbd gt 0 then mask[w[w2]] = 0
;    tempmask = where(mask eq 0,nbd)
;   endif else begin 
;    mask = im*0.+1.
;    mask[tempmask]=0.
;    nbd = n_elements(tempmask)
;   endelse

;   med = median(im)
;   if nbd gt 0 then im[tempmask] = 0
 
;   irac_expand,im,hdr,mask,70
;   kx = fltarr(3,3) & ky = fltarr(3,3)
;   kx[0,1] = 1. & ky[1,0] = 1.

;   if not keyword_set(nodistcorr) then irac_dist,im,hdr,mask,kx,ky,70
;
;   if keyword_set(derotate) then irac_rot,im,hdr,mask,kx,ky,rot1

;   xoff = xoffsets[i,j] - xoffsets[0,j]
;   yoff = yoffsets[i,j] - yoffsets[0,j]
 
;   tkx = kx & tky = ky

;   irac_ccrefine,master,mmask,im,mask,xoff,yoff,kx,ky

 ;  newxoff = xoff+tkx[0,0]-kx[0,0]
 ;  newyoff = yoff+tky[0,0]-ky[0,0]

 ;  xoffsets[i,j] = xoffsets[0,j]+newxoff
 ;  yoffsets[i,j] = yoffsets[0,j]+newyoff
;   xoffsets[i,j] = xoffsets[0,j]+xoff
;   yoffsets[i,j] = yoffsets[0,j]+yoff

;   delvarx,kx,ky

;  endfor

;merge submosaic
;  txoff = 0 & tyoff = 0
;  for i=1,nmos-1 do begin
   
;   fits_datmask,f[i,j],im,hdr,tempmask

;   if tempmask[0] eq -1 then begin
;    mask = finite(im)
;    w=where(mask eq 1,ngd)
;    if ngd gt 0 then w2 = where(im[w] lt 0,nbd)
;    if nbd gt 0 then mask[w[w2]] = 0
;    tempmask = where(mask eq 0,nbd)
;   endif else begin
;    mask = im*0.+1.
;    mask[tempmask]=0.
;    nbd = n_elements(tempmask)
;   endelse

;   med = median(im)
;   if nbd gt 0 then im[tempmask] = 0

;   irac_expand,im,hdr,mask,70
;   kx = fltarr(3,3) & ky = fltarr(3,3)
;   kx[0,1] = 1. & ky[1,0] = 1.

;   if not keyword_set(nodistcorr) then irac_dist,im,hdr,mask,kx,ky,70

;   if keyword_set(derotate) then irac_rot,im,hdr,mask,kx,ky,rot1

;   xoff = xoffsets[i,j] - xoffsets[0,j] + txoff

;   yoff = yoffsets[i,j] - yoffsets[0,j] + tyoff

;   irac_spoffset,im,hdr,mask,kx,ky,xoff,yoff

;   im = poly_2d(temporary(im),kx,ky,1,missing=0)
;   mask = poly_2d(temporary(mask),kx,ky,1,missing=0)

;   tempmask = where(mask ge .1,ngd)
;   if ngd gt 0 then im[tempmask] = im[tempmask] / mask[tempmask]
;   tempmask = where(mask lt .1,nbd)
;   if nbd gt 0 then begin
;    im[tempmask] = med
;    mask[tempmask] = 0
;    nbd = n_elements(tempmask)
;   endif

;   mosaic_two,master,mmask,im,mask,xoff,yoff,xmoff,ymoff,/autoffset
 
;   delvarx,kx,ky

;   erase&tvimage,bytscl(master,min=0,max=4),/keep

;stop

;   txoff = txoff + xmoff
;   tyoff = tyoff + ymoff
;  endfor

;save master submosaic 
;  if j eq 0 then begin
;   tmhdr = mhdr
;   tmaster = master
;   tmmask = mmask
;   txmoff = txoff
;   tymoff = tyoff

;else refine submosaic offset to master submos and shove it in for now.
;  endif else begin
    
;   xoff = xoffsets[0,j] - txoff + txmoff
;   yoff = yoffsets[0,j] - tyoff + tymoff
;
;   print,'Before: ',xoff,yoff
;   irac_smrefine,tmaster,tmmask,master,mmask,xoff,yoff
;   print,'After: ',xoff,yoff

;   xoffsets[*,j] = xoffsets[*,j] + xoff - (xoffsets[0,j] - txoff + txmoff)
;   yoffsets[*,j] = yoffsets[*,j] + yoff - (yoffsets[0,j] - tyoff + tymoff)

;   mosaic_two,tmaster,tmmask,master,mmask,xoff,yoff,xmoff,ymoff,/autoffset

;   erase&tvimage,bytscl(tmaster,min=0,max=4),/keep

;   stop

;   txmoff = txmoff + xmoff
;   tymoff = tymoff + ymoff

;  endelse
; endfor

; endif

;----------------------------------

;One more time, with all offsets now refined, updating WCS info as we go
; for the end-product mosaic, which we can write to working dir along with 
; offsets, filenames, etc.

 txoff = 0 & tyoff = 0
 for j = 0,npm-1 do begin

  for i = 0,nmos-1 do begin

   fits_datmask,f[i,j],im,hdr,tempmask

   if tempmask[0] eq -1 then begin
    mask = finite(im)
    w=where(mask eq 1,ngd)
    if ngd gt 0 then w2 = where(im[w] lt 0,nbd)
    if nbd gt 0 then mask[w[w2]] = 0
    tempmask = where(mask eq 0,nbd)
   endif else begin
    mask = im*0.+1.
    mask[tempmask]=0.
    nbd = n_elements(tempmask)
   endelse

   med = median(im)
   if nbd gt 0 then im[tempmask] = 0

   irac_expand,im,hdr,mask,70
   kx = fltarr(3,3) & ky = fltarr(3,3)
   kx[0,1] = 1. & ky[1,0] = 1.

   if not keyword_set(nodistcorr) then irac_dist,im,hdr,mask,kx,ky,70

   if keyword_set(derotate) then irac_rot,im,hdr,mask,kx,ky,rot1

   if i+j eq 0 then begin
    master = poly_2d(temporary(im)*mask,kx,ky,1,missing=0)
    mmask = poly_2d(temporary(mask),kx,ky,1,missing=0)
    mhdr = temporary(hdr)
    tempmask = where(mmask ge .1,ngd)
    if ngd gt 0 then master[tempmask] = master[tempmask] / mmask[tempmask]
    tempmask = where(mmask lt .1,nbd)
    if nbd gt 0 then begin
     master[tempmask] = med
     mmask[tempmask] = 0
     nbd = n_elements(tempmask)
    endif
   endif else begin

    xoff = xoffsets[i,j] + txoff

    yoff = yoffsets[i,j] + tyoff

    irac_spoffset,im,hdr,mask,kx,ky,xoff,yoff

    im = poly_2d(temporary(im)*mask,kx,ky,1,missing=0)
    mask = poly_2d(temporary(mask),kx,ky,1,missing=0)

    tempmask = where(mask ge .1,ngd)
    if ngd gt 0 then im[tempmask] = im[tempmask] / mask[tempmask]
    tempmask = where(mask lt .1,nbd)
    if nbd gt 0 then begin
     im[tempmask] = med
     mask[tempmask] = 0
     nbd = n_elements(tempmask)
    endif

    mosaic_two,master,mmask,im,mask,xoff,yoff,xmoff,ymoff,/autoffset

    delvarx,kx,ky

;    erase&tvimage,bytscl(master,min=0,max=4),/keep

    txoff = txoff + xmoff
    tyoff = tyoff + ymoff

    oldx = sxpar(mhdr,'CRPIX1')
    oldy = sxpar(mhdr,'CRPIX2')

    sxaddpar,mhdr,'CRPIX1',oldx[0]+xmoff[0]
    sxaddpar,mhdr,'CRPIX2',oldy[0]+ymoff[0]
   endelse
  endfor
 endfor

;no matter what, make sure distortion keywords get ignored in mosaics
 sxaddpar,mhdr,'CTYPE1',strmid(strtrim(sxpar(mhdr,'CTYPE1'),2),0,8)
 sxaddpar,mhdr,'CTYPE2',strmid(strtrim(sxpar(mhdr,'CTYPE2'),2),0,8)

 fits_datmask,/write,outfiles[c],master,mhdr,mmask

 if hdr_type eq 0 and hdr_mode eq 1 then irac_hdr_merge,longfile=outfiles[c-1],shortfile=outfiles[c],outfile=workdir+'ch'+strtrim(chan,2)+'_hdr_mosaic.fits';,nosip=nosip

 endfor

; out = {mosaic:master,master:mmask,hdr:mhdr} 

; return,out

end
